  /*
  Description:
  Testing DS1868 digital potentiometer with PICMicro
  MCU: PIC16F1827 @ 16 MHz external ceramic resonator, MCLR Disabled
  The H and L terminals of potentiometer are connected to Vcc
  and Gnd, whereas the voltages at wiper terminals are read through
  RA0 and RA1 analog channels.
  
  Copyright @ Rajendra Bhatt
  Date: Jun 21, 2011
*/

// Define LCD module connections.
 sbit LCD_RS at RB0_bit;
 sbit LCD_EN at RB1_bit;
 sbit LCD_D4 at RB4_bit;
 sbit LCD_D5 at RB5_bit;
 sbit LCD_D6 at RB6_bit;
 sbit LCD_D7 at RB7_bit;
 sbit LCD_RS_Direction at TRISB0_bit;
 sbit LCD_EN_Direction at TRISB1_bit;
 sbit LCD_D4_Direction at TRISB4_bit;
 sbit LCD_D5_Direction at TRISB5_bit;
 sbit LCD_D6_Direction at TRISB6_bit;
 sbit LCD_D7_Direction at TRISB7_bit;
// End LCD module connection definition

sbit Reset at RA4_bit;
sbit Data at RB2_bit;
sbit Clock at RB3_bit;
sbit Up at RA2_bit;
sbit Down at RA3_bit;

//POT values;
unsigned short aa;

char message1[] = "V1=";
char message2[] = "V2=";
char *ADC = "0000";

void Send_POT_Value(unsigned short num1){
 unsigned short t, Flag, Mask;
 
 Mask = 0x80;
 Reset = 1;    // Pull Reset line HIGH to initiate Data transfer
 Data = 0;     // Stack select bit 0
 Clock = 1;
 Delay_us(1);
 Clock = 0;
 
 // Shift in 8-bit wiper position for POT1
 for (t=0; t<8; t++){
   Flag = num1 & Mask;
   if(Flag==0) Data = 0;
   else Data = 1;
   Clock = 1;
   Delay_us(1);
   Clock = 0;
   Mask = Mask >> 1;
 }
 // Shift in 8-bit wiper position for POT0
 Mask = 0x80;
 for (t=0; t<8; t++){
  Flag = num1 & Mask;
  if(Flag==0) Data = 0;
  else Data = 1;
  Clock = 1;
  Delay_us(1);
  Clock = 0;
  Mask = Mask >> 1;
 }
 Reset = 0;   // End Data transfer
}

void Debounce_Key(){
 Delay_ms(300);
}

void ADC_Display(unsigned short ch, unsigned short row){
 unsigned int ADC_Value;
 ADC_Value = ADC_Read(ch);
 ADC[0] = ADC_Value/1000 + 48;
 ADC[1] = (ADC_Value/100)%10 + 48;
 ADC[2] = (ADC_Value/10)%10 + 48;
 ADC[3] = ADC_Value%10 + 48;
 LCD_Out(row,4, ADC );
}

void main() {
 ANSELA = 0b00000011;  // RA0, RA1 analog inputs
 TRISA  = 0b00101111;  // RA0-RA3 inputs
 ANSELB = 0x00;        // PORTB all digital I/O
 TRISB  = 0x00;
 
 Lcd_Init();        // Initialize LCD
 Lcd_Cmd(_LCD_CLEAR);             // CLEAR display
 Lcd_Cmd(_LCD_CURSOR_OFF);        // Cursor off
 Lcd_Out(1,1,message1);            // Write message1 in 1st row
 Lcd_Out(2,1,message2);            // Write message2 in 2nd row
 aa = 0;
 Send_POT_Value(0);
 ADC_Display(0, 1);
 ADC_Display(1, 2);
 do {
  if (!Up) {
   Debounce_Key();
   if (aa < 255) {
    aa = aa + 5;
   }
   Send_POT_Value(aa);
   ADC_Display(0, 1);
   ADC_Display(1, 2);

  }

  if (!Down) {
   Debounce_Key();
   if (aa > 0) {
    aa = aa - 5;
   }
   Send_POT_Value(aa);
   ADC_Display(0, 1);
   ADC_Display(1, 2);
  }
  

 } while (1);



}